/*******************************************************************************
 System Interrupts File

  Company:
    Microchip Technology Inc.

  File Name:
    interrupt.c

  Summary:
    Interrupt vectors mapping

  Description:
    This file maps all the interrupt vectors to their corresponding
    implementations. If a particular module interrupt is used, then its ISR
    definition can be found in corresponding PLIB source file. If a module
    interrupt is not used, then its ISR implementation is mapped to dummy
    handler.
 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
* Copyright (C) 2018 Microchip Technology Inc. and its subsidiaries.
*
* Subject to your compliance with these terms, you may use Microchip software
* and any derivatives exclusively with Microchip products. It is your
* responsibility to comply with third party license terms applicable to your
* use of third party software (including open source software) that may
* accompany Microchip software.
*
* THIS SOFTWARE IS SUPPLIED BY MICROCHIP "AS IS". NO WARRANTIES, WHETHER
* EXPRESS, IMPLIED OR STATUTORY, APPLY TO THIS SOFTWARE, INCLUDING ANY IMPLIED
* WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY, AND FITNESS FOR A
* PARTICULAR PURPOSE.
*
* IN NO EVENT WILL MICROCHIP BE LIABLE FOR ANY INDIRECT, SPECIAL, PUNITIVE,
* INCIDENTAL OR CONSEQUENTIAL LOSS, DAMAGE, COST OR EXPENSE OF ANY KIND
* WHATSOEVER RELATED TO THE SOFTWARE, HOWEVER CAUSED, EVEN IF MICROCHIP HAS
* BEEN ADVISED OF THE POSSIBILITY OR THE DAMAGES ARE FORESEEABLE. TO THE
* FULLEST EXTENT ALLOWED BY LAW, MICROCHIP'S TOTAL LIABILITY ON ALL CLAIMS IN
* ANY WAY RELATED TO THIS SOFTWARE WILL NOT EXCEED THE AMOUNT OF FEES, IF ANY,
* THAT YOU HAVE PAID DIRECTLY TO MICROCHIP FOR THIS SOFTWARE.
 *******************************************************************************/
// DOM-IGNORE-END

// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************
#include "configuration.h"
#include "interrupts.h"
#include "definitions.h"



// *****************************************************************************
// *****************************************************************************
// Section: System Interrupt Vector Functions
// *****************************************************************************
// *****************************************************************************


/* All the handlers are defined here.  Each will call its PLIB-specific function. */
// *****************************************************************************
// *****************************************************************************
// Section: System Interrupt Vector declarations
// *****************************************************************************
// *****************************************************************************
void CORE_TIMER_Handler (void);
void SPI1_RX_Handler (void);
void SPI1_TX_Handler (void);
void I2C1_BUS_Handler (void);
void I2C1_MASTER_Handler (void);
void DMA0_Handler (void);
void DMA1_Handler (void);
void ETHERNET_Handler (void);
void SPI3_RX_Handler (void);
void SPI3_TX_Handler (void);
void RTCC_Handler (void);
void UART6_FAULT_Handler (void);
void UART6_RX_Handler (void);
void UART6_TX_Handler (void);


// *****************************************************************************
// *****************************************************************************
// Section: System Interrupt Vector definitions
// *****************************************************************************
// *****************************************************************************
void CORE_TIMER_Handler (void)
{
    CORE_TIMER_InterruptHandler();
}



void SPI1_RX_Handler (void)
{
    SPI1_RX_InterruptHandler();
}

void SPI1_TX_Handler (void)
{
    SPI1_TX_InterruptHandler();
}

void I2C1_BUS_Handler (void)
{
    I2C1_BUS_InterruptHandler();
}

void I2C1_MASTER_Handler (void)
{
    I2C1_MASTER_InterruptHandler();
}

void DMA0_Handler (void)
{
    DMA0_InterruptHandler();
}

void DMA1_Handler (void)
{
    DMA1_InterruptHandler();
}

void ETHERNET_Handler (void)
{
    ETHERNET_InterruptHandler();
}

void SPI3_RX_Handler (void)
{
    SPI3_RX_InterruptHandler();
}

void SPI3_TX_Handler (void)
{
    SPI3_TX_InterruptHandler();
}

void RTCC_Handler (void)
{
    RTCC_InterruptHandler();
}

void UART6_FAULT_Handler (void)
{
    UART6_FAULT_InterruptHandler();
}

void UART6_RX_Handler (void)
{
    UART6_RX_InterruptHandler();
}

void UART6_TX_Handler (void)
{
    UART6_TX_InterruptHandler();
}




/*******************************************************************************
 End of File
*/
